# coding: utf-8

require 'rb_conclas/core/requester'
require 'rb_conclas/core/exceptions/client_exceptions'
require 'rb_conclas/core/conclas_response'
require 'rb_conclas/utils/json_maker'
require 'rb_conclas/utils/string_helper'
require 'json'

BASE_URL = "http://localhost:5000/v1"
METHODS = {
  :direct => "/categorise/direct/",
  :indirect => "/categorise/indirect/"
}

module Conclas
  #module to abstract request
  class Api
    #include methods of class Request
    include Core::Requester

    def initialize(token, timeout=60, use_ssl=false)
      @timeout = timeout
      if Utils::StringHelper.is_empty token
        raise EmptyParameterException, "The parameter 'token' needs a value"
      end
      @token = token
      @use_ssl = use_ssl
      @status_code = nil
      @headers = nil
      @result = nil
      @req = nil
    end

    #properties
    def status_code
      @status_code
    end

    def headers
      @headers
    end

    def result
      @result
    end

    #private methods
    def make_request(url, data)
      headers = {"Content-Type" => "application/json",
                  "Authorization" => @token}
      @req = Core::Requester.post(url, data, headers, @timeout, @use_ssl)

      if check_request

        response = Core::ConclasResponse.new(
          @req.body, 
          @req.code, 
          @req.header.to_hash.inspect)

        if response.request_successful?
          populate_properties
        end
      end
    end

    def check_request
      if @req.code.to_i.between?(400, 499)
        raise Exceptions::HTTPError, "Client Error #{@req.code}(#{@req.message})."
      elsif @req.code.to_i.between?(500, 600)
        raise Exceptions::HTTPError, "Server Error #{@req.code}(#{@req.message})."
      else
        true
      end
    end

    def populate_properties
      @status_code = @req.code
      @headers = @req.header.to_hash.inspect
      @result = JSON.parse(@req.body, :quirks_mode => true)
    end

    def check_api_args_method(contents, start_from_category=nil, callback=nil)
      if contents.length == 0 || !contents.is_a?(Array)
        raise InvalidParameterException, "The parameter 'contents' needs a value"
      end
      
      if contents.length > 1000
        raise DocMaxException, "The content size exceeded the limit."
      end
      #if !start_from_category.nil?
      #  if Utils::StringHelper.is_empty start_from_category
      #    raise Exceptions::InvalidParameterException, "The parameter 'start_from_category' needs a value"
      #  end
      #end

      if !callback.nil?
        if Utils::StringHelper.is_empty callback
          raise Exceptions::InvalidParameterException, "The parameter 'callback' needs a value"
        end
      end
    end

    private :populate_properties, :check_api_args_method, :make_request


    #main methods
    def direct_categorise(contents)
      check_api_args_method(contents)
      hashe_data = {:contents => contents}
      #if !start_from_category.nil?
      #  hashe_data[:start_from_category] = start_from_category
      #end
      json_body_request = Utils::JsonMaker.new(hashe_data).get_json
      url = BASE_URL + METHODS[:direct]
      make_request(url, json_body_request)
    end
    
    def indirect_categorise(contents, callback)
      check_api_args_method(contents, callback)
      hashe_data = {:contents => contents, :callback => callback}
      #if !start_from_category.nil?
      #  hashe_data[:start_from_category] = start_from_category
      #end
      json_body_request = Utils::JsonMaker.new(hashe_data).get_json
      url = BASE_URL + METHODS[:indirect]
      make_request(url, json_body_request)
    end
  end
end